package com.ruoyi.framework.web.service;

import javax.annotation.Resource;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.system.domain.TDoctor;
import com.ruoyi.system.domain.req.AppRegisterReq;
import com.ruoyi.system.mapper.TDoctorMapper;
import com.ruoyi.system.service.impl.UserTokenService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.user.CaptchaException;
import com.ruoyi.common.exception.user.CaptchaExpireException;
import com.ruoyi.common.exception.user.UserPasswordNotMatchException;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.framework.manager.AsyncManager;
import com.ruoyi.framework.manager.factory.AsyncFactory;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysUserService;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 登录校验方法
 * 
 * @author ruoyi
 */
@Component
public class SysLoginService
{
    private static final Logger log = LoggerFactory.getLogger(SysLoginService.class);

    @Value("${ruoyi.profile}")
    private String uploadFilePath;
    @Autowired
    private TokenService tokenService;

    @Resource
    private AuthenticationManager authenticationManager;

    @Autowired
    private RedisCache redisCache;
    
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Resource
    private TDoctorMapper tDoctorMapper;

    @Resource
    private UserTokenService userTokenService;

    /**
     * 登录验证
     * 
     * @param username 用户名
     * @param password 密码
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public String login(String username, String password, String code, String uuid)
    {
        boolean captchaOnOff = configService.selectCaptchaOnOff();
        // 验证码开关
        if (captchaOnOff)
        {
            validateCaptcha(username, code, uuid);
        }
        // 用户验证
        Authentication authentication = null;
        try
        {
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager
                    .authenticate(new UsernamePasswordAuthenticationToken(username, password));
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordLoginInfo(loginUser.getUserId());
        // 生成token
        return tokenService.createToken(loginUser);
    }

    public AjaxResult loginForApp(String username, String password) {
        AjaxResult ajax = AjaxResult.success();
        // 查询用户是否注册过 或者注册过已被删除
        TDoctor tDoctor = tDoctorMapper.selectTDoctorByPhone(username);
        if (Objects.isNull(tDoctor) || "2".equals(tDoctor.getDelFlag())) {
            // 未注册或已删除 返回flag app跳完善信息页面
            ajax.put("flag", false);
        } else {
            SysUser sysUser = userService.selectUserByUserName(username);
            // 验证密码
            if (!SecurityUtils.matchesPassword(password, sysUser.getPassword())) {
                log.info("登录用户：{} 密码不正确.", username);
                throw new ServiceException("登录失败！");
            }
            LoginUser loginUser = new LoginUser();
            sysUser.setDoctorId(tDoctor.getId());
            sysUser.setHospitalId(tDoctor.getHospitalId());
            loginUser.setUserId(sysUser.getUserId());
            loginUser.setUser(sysUser);
            loginUser.setRequestFrom("1");
            String token = userTokenService.createTokenForApp(loginUser);
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(sysUser.getUserName(), Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
            recordLoginInfo(sysUser.getUserId());
            ajax.put(Constants.TOKEN, token);
            ajax.put("flag", true);
        }
        ajax.put(Constants.REQUEST_FROM, "app");
        return ajax;
    }

    public AjaxResult loginForAppFillInfo(AppRegisterReq req) {
        if (StringUtils.isNotEmpty(req.getMedicalLicense())) {
            log.info("MedicalLicense is not null.");
            req.setMedicalLicense(FileUploadUtils.generateImage(req.getMedicalLicense(), uploadFilePath));
        }
        log.info("login for app fill info req :{}", req);
        // 查询sys_user数据
        SysUser sysUser = userService.selectUserByPhone(req.getPhone());
        if (Objects.nonNull(sysUser)) {
            if ("2".equals(sysUser.getDelFlag())) {
                // 医生账号被删除 重新注册
                sysUser.setDelFlag("0");
                sysUser.setPassword(SecurityUtils.encryptPassword("123456"));
                sysUser.setUserName(req.getPhone());
                sysUser.setNickName(req.getName());
                sysUser.setStatus("0");
                userService.updateUser(sysUser);
            }
        } else {
            // 注册
            sysUser = new SysUser();
            sysUser.setPhonenumber(req.getPhone());
            sysUser.setUserName(req.getPhone());
            sysUser.setNickName(req.getName());
            sysUser.setStatus("0");
            sysUser.setDelFlag("0");
            sysUser.setPassword(SecurityUtils.encryptPassword("123456"));
            Long[] roles = new Long[]{3L};
            sysUser.setRoleIds(roles);
            userService.insertUser(sysUser);
        }
        // 查询医生表数据
        TDoctor tDoctor = tDoctorMapper.selectTDoctorByPhone(req.getPhone());
        if (Objects.nonNull(tDoctor)) {
            if ("2".equals(tDoctor.getDelFlag())) {
                // 重新注册
                tDoctor.setDelFlag("0");
                tDoctor.setUpdateTime(new Date());
                tDoctor.setHospitalId(req.getHospitalId());
                tDoctor.setName(req.getName());
                tDoctor.setSpeciality(req.getSpeciality());
                tDoctor.setMedicalLicense(req.getMedicalLicense());
                tDoctor.setUserId(sysUser.getUserId());
                tDoctorMapper.updateTDoctor(tDoctor);
            }
        } else {
            // 注册
            tDoctor = new TDoctor();
            tDoctor.setDelFlag("0");
            tDoctor.setPhone(req.getPhone());
            tDoctor.setUpdateTime(new Date());
            tDoctor.setCreateTime(new Date());
            tDoctor.setHospitalId(req.getHospitalId());
            tDoctor.setName(req.getName());
            tDoctor.setSpeciality(req.getSpeciality());
            tDoctor.setMedicalLicense(req.getMedicalLicense());
            tDoctor.setUserId(sysUser.getUserId());
            tDoctorMapper.insertTDoctor(tDoctor);
            tDoctor.setIdentifier("10000000" + tDoctor.getId());
            Map map = new HashMap();
            map.put("id", tDoctor.getId());
            map.put("from", "yinuo");
            String content = JSONObject.toJSONString(map);
            String qrCode = FileUploadUtils.code(content, uploadFilePath);
            tDoctor.setQrCode(qrCode);
            log.info("update doctor :{}", tDoctor);
            tDoctorMapper.updateTDoctor(tDoctor);
        }
        LoginUser loginUser = new LoginUser();
        sysUser.setDoctorId(tDoctor.getId());
        sysUser.setHospitalId(tDoctor.getHospitalId());
        loginUser.setUserId(sysUser.getUserId());
        loginUser.setUser(sysUser);
        loginUser.setRequestFrom("1");
        String token = userTokenService.createTokenForApp(loginUser);
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(sysUser.getUserName(), Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        AjaxResult ajaxResult = AjaxResult.success();
        ajaxResult.put(Constants.TOKEN, token);
        return ajaxResult;
    }

    private String buildTokenByDoctor(SysUser sysUser) {
        TDoctor tDoctor = tDoctorMapper.selectTDoctorByUserId(sysUser.getUserId());
        if (Objects.isNull(tDoctor)) {
            log.info("登录用户：{} 不存在.", sysUser.getUserName());
            throw new ServiceException("登录用户：" + sysUser.getUserName() + " 不存在，请先注册！");
        }
        LoginUser loginUser = new LoginUser();
        loginUser.setUserId(sysUser.getUserId());
        loginUser.setUser(sysUser);
        String token = userTokenService.createTokenForApp(loginUser);
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(sysUser.getUserName(), Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        return token;
    }

    private SysUser checkLoginUser(String username, String password) {
        SysUser user = userService.selectUserByUserName(username);
        if (StringUtils.isNull(user))
        {
            log.info("登录用户：{} 不存在.", username);
            throw new ServiceException("登录用户：" + username + " 不存在，请先注册！");
        }
        else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
        {
            log.info("登录用户：{} 已被删除.", username);
            throw new ServiceException("对不起，您的账号：" + username + " 已被删除");
        }
        else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
        {
            log.info("登录用户：{} 已被停用.", username);
            throw new ServiceException("对不起，您的账号：" + username + " 已停用");
        } else if (!SecurityUtils.matchesPassword(password, user.getPassword())) {
            log.info("登录用户：{} 密码不正确.", username);
            throw new ServiceException("登录失败！");
        }
        return user;
    }

    /**
     * 校验验证码
     * 
     * @param username 用户名
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        String verifyKey = Constants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
        String captcha = redisCache.getCacheObject(verifyKey);
        redisCache.deleteObject(verifyKey);
        if (captcha == null)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
            throw new CaptchaExpireException();
        }
        if (!code.equalsIgnoreCase(captcha))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
            throw new CaptchaException();
        }
    }

    /**
     * 记录登录信息
     *
     * @param userId 用户ID
     */
    public void recordLoginInfo(Long userId)
    {
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        sysUser.setLoginIp(IpUtils.getIpAddr(ServletUtils.getRequest()));
        sysUser.setLoginDate(DateUtils.getNowDate());
        userService.updateUserProfile(sysUser);
    }
}
