package com.ruoyi.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.common.utils.wechat.WechatKeys;
import com.ruoyi.system.domain.TDoctor;
import com.ruoyi.system.domain.TPatient;
import com.ruoyi.system.domain.TWechatUser;
import com.ruoyi.system.domain.req.GetOpenidReq;
import com.ruoyi.system.domain.req.LoginForAppReq;
import com.ruoyi.system.domain.req.LoginForMpReq;
import com.ruoyi.system.domain.resp.GetOpenidResp;
import com.ruoyi.system.mapper.TDoctorMapper;
import com.ruoyi.system.mapper.TPatientMapper;
import com.ruoyi.system.mapper.TWechatUserMapper;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.service.IWechatService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@Service
public class WechatServiceImpl implements IWechatService {
    private static final Logger logger = LoggerFactory.getLogger(WechatServiceImpl.class);
    /**
     * 微信小程序AppID
     */
    @Value("${wx.mpAppId}")
    private String appId;
    /**
     * 微信小程序AppSecret
     */
    @Value("${wx.mpSecret}")
    private String appSecret;

    @Value("${wx.wechatAppId}")
    private String wechatAppId;
    /**
     * 微信小程序AppSecret
     */
    @Value("${wx.wechatSecret}")
    private String wechatSecret;

    @Value("${ruoyi.profile}")
    private String uploadFilePath;

    @Resource
    private TWechatUserMapper tWechatUserMapper;

    @Resource
    private TPatientMapper tPatientMapper;

    @Resource
    private TDoctorMapper tDoctorMapper;

    @Resource
    private UserTokenService userTokenService;

    @Autowired
    private ISysUserService userService;

    @Override
    public GetOpenidResp getOpenid(GetOpenidReq req, HttpServletRequest request) {
        //查询openid
        String openid = queryOpenid(req);
        TWechatUser wechatUser = updateWechatUserByOpenid(openid, req, request);
        GetOpenidResp resp = new GetOpenidResp();
        if (Objects.isNull(wechatUser.getUserId())) {
            resp.setBindFlag(false);
        } else {
            // 已绑定信息 直接登录
            resp.setBindFlag(true);
            TPatient tPatient = tPatientMapper.selectTPatientById(wechatUser.getUserId());
            String token = buildTokenByPatient(tPatient);
            resp.setToken(token);
        }
        resp.setOpenid(openid);
        return resp;
    }

    @Override
    public GetOpenidResp getUnionid(GetOpenidReq req, HttpServletRequest request) {
        //查询unionid
        String unionid = queryUnionid(req);
        TWechatUser wechatUser = updateWechatUserByUnionid(unionid, req, request);
        GetOpenidResp resp = new GetOpenidResp();
        if (Objects.isNull(wechatUser.getUserId())) {
            resp.setBindFlag(false);
        } else {
            // 已绑定信息 直接登录
            resp.setBindFlag(true);
            TDoctor tDoctor = tDoctorMapper.selectTDoctorById(wechatUser.getUserId());
            String token = buildTokenByDoctor(tDoctor);
            resp.setToken(token);
        }
        resp.setUnionid(unionid);
        return resp;
    }

    @Override
    public String loginForMp(LoginForMpReq req) {
        // 查询手机号是否存在患者信息
        TPatient tPatient = new TPatient();
        BeanUtils.copyProperties(req, tPatient);
        TPatient currentPatient = tPatientMapper.queryTPatient(tPatient);
        TWechatUser wechatUser = new TWechatUser();
        if (Objects.nonNull(currentPatient)) {
            // 存在 需要把患者信息更新 并绑定openid
            BeanUtils.copyProperties(req, currentPatient);
            currentPatient.setDelFlag("0");
            tPatientMapper.updateTPatient(currentPatient);
            wechatUser.setUserId(currentPatient.getId());
            tPatient.setId(currentPatient.getId());
        } else {
            // 不存在 患者信息入库
            tPatient.setDelFlag("0");
            tPatient.setCreateTime(new Date());
            tPatient.setUpdateTime(new Date());
            tPatientMapper.insertTPatient(tPatient);
            tPatient.setIdentifier("10000000" + tPatient.getId());
            tPatientMapper.updateTPatient(tPatient);
            wechatUser.setUserId(tPatient.getId());
        }
        // 绑定openid
        wechatUser.setOpenid(req.getOpenId());
        tWechatUserMapper.updateWechatUserByOpenId(wechatUser);
        // 登录获取token
        String token = buildTokenByPatient(tPatient);
        return token;
    }

    @Override
    public String loginForApp(LoginForAppReq req) {
        // 查询手机号是否存在医生信息
        TDoctor tDoctor = new TDoctor();
        if (StringUtils.isNotEmpty(req.getMedicalLicense())) {
            req.setMedicalLicense(FileUploadUtils.generateImage(req.getMedicalLicense(), uploadFilePath));
        }
        BeanUtils.copyProperties(req, tDoctor);
        TDoctor currentDoctor = tDoctorMapper.selectTDoctorByPhone(req.getPhone());
        TWechatUser wechatUser = new TWechatUser();
        if (Objects.nonNull(currentDoctor)) {
            TWechatUser queryUser = tWechatUserMapper.selectWechatUserByUserId(currentDoctor.getId());
            if (Objects.nonNull(queryUser)) {
                throw new ServiceException("该手机号已绑定其他微信账号！");
            }
            // 存在 需要把患者信息更新 并绑定openid
            BeanUtils.copyProperties(req, currentDoctor);
            currentDoctor.setDelFlag("0");
            tDoctorMapper.updateTDoctor(currentDoctor);
            wechatUser.setUserId(currentDoctor.getId());
            tDoctor.setId(currentDoctor.getId());
        } else {
            SysUser sysUser = new SysUser();
            sysUser.setPhonenumber(req.getPhone());
            sysUser.setUserName(req.getPhone());
            sysUser.setNickName(req.getName());
            sysUser.setStatus("0");
            sysUser.setDelFlag("0");
            sysUser.setPassword(SecurityUtils.encryptPassword("123456"));
            Long[] roles = new Long[]{3L};
            sysUser.setRoleIds(roles);
            userService.insertUser(sysUser);
            // 不存在 医生信息入库
            tDoctor.setDelFlag("0");
            tDoctor.setCreateTime(new Date());
            tDoctor.setUpdateTime(new Date());
            tDoctor.setUserId(sysUser.getUserId());
            tDoctorMapper.insertTDoctor(tDoctor);
            tDoctor.setIdentifier("10000000" + tDoctor.getId());

            Map map = new HashMap();
            map.put("id", tDoctor.getId());
            map.put("from", "yinuo");
            String content = JSONObject.toJSONString(map);
            String qrCode = FileUploadUtils.code(content, uploadFilePath);
            tDoctor.setQrCode(qrCode);
            logger.info("update doctor :{}", tDoctor);
            tDoctorMapper.updateTDoctor(tDoctor);
            wechatUser.setUserId(tDoctor.getId());

        }
        // 绑定openid
        wechatUser.setUnionid(req.getUnionid());
        tWechatUserMapper.updateWechatUserByUnionId(wechatUser);
        // 登录获取token
        String token = buildTokenByDoctor(tDoctor);
        return token;
    }

    private String queryOpenid(GetOpenidReq req){
        String url = String.format(WechatKeys.MP_CODE_TO_OPENID_URL, appId, appSecret, req.getCode());
        String json = HttpUtils.sendGet(url);
        JSONObject jsonObject = JSONObject.parseObject(json);
        String session_key = (String) jsonObject.get("session_key");
        String openid = (String) jsonObject.get("openid");
        logger.info("login openid:{},session_key:{}", openid, session_key);
        if (StringUtils.isEmpty(openid)) {
            throw new ServiceException("未获取到openid");
        }
        return openid;
    }

    private String queryUnionid(GetOpenidReq req){
        String url = String.format(WechatKeys.WECHAT_CODE_TO_UNIONID_URL, wechatAppId, wechatSecret, req.getCode());
        String json = HttpUtils.sendGet(url);
        JSONObject jsonObject = JSONObject.parseObject(json);
        logger.info("login wechat user info :{}", jsonObject.toJSONString());
        String unionid = (String) jsonObject.get("unionid");
        logger.info("login openid:{}", unionid);
        if (StringUtils.isEmpty(unionid)) {
            throw new ServiceException("未获取到unionid");
        }
        return unionid;
    }

    private TWechatUser updateWechatUserByOpenid(String openid, GetOpenidReq req, HttpServletRequest request){
        //查询当前微信用户是否存在
        TWechatUser wechatUser = tWechatUserMapper.selectWechatUserByOpenid(openid);
        if (wechatUser == null){
            wechatUser = new TWechatUser();
            wechatUser.setOpenid(openid);
            wechatUser.setAvatar(req.getAvatarUrl());
            wechatUser.setNickname(req.getNickName());
            wechatUser.setLastLoginTime(new Date());
            wechatUser.setLastLoginIp(IpUtils.getIpAddr(request));
            wechatUser.setStatus((byte) 0);
            tWechatUserMapper.insert(wechatUser);

        }
        wechatUser.setLastLoginTime(new Date());
        wechatUser.setLastLoginIp(IpUtils.getIpAddr(request));
        tWechatUserMapper.updateByPrimaryKeySelective(wechatUser);
        return wechatUser;
    }

    private TWechatUser updateWechatUserByUnionid(String unionid, GetOpenidReq req, HttpServletRequest request){
        //查询当前微信用户是否存在
        TWechatUser wechatUser = tWechatUserMapper.selectWechatUserByUnionid(unionid);
        if (wechatUser == null){
            wechatUser = new TWechatUser();
            wechatUser.setUnionid(unionid);
            wechatUser.setAvatar(req.getAvatarUrl());
            wechatUser.setNickname(req.getNickName());
            wechatUser.setLastLoginTime(new Date());
            wechatUser.setLastLoginIp(IpUtils.getIpAddr(request));
            wechatUser.setStatus((byte) 0);
            tWechatUserMapper.insert(wechatUser);

        }
        wechatUser.setLastLoginTime(new Date());
        wechatUser.setLastLoginIp(IpUtils.getIpAddr(request));
        tWechatUserMapper.updateByPrimaryKeySelective(wechatUser);
        return wechatUser;
    }

    private String buildTokenByPatient(TPatient tPatient) {
        if (Objects.isNull(tPatient)) {
            return StringUtils.EMPTY;
        }
        LoginUser loginUser = new LoginUser();
        loginUser.setUserId(tPatient.getId());
        SysUser user = new SysUser();
        user.setUserId(tPatient.getId());
        user.setUserName(tPatient.getName());
        user.setDelFlag(tPatient.getDelFlag());
        user.setSex(tPatient.getSex());
        user.setPhonenumber(tPatient.getPhone());
        loginUser.setUser(user);
        String token = userTokenService.createTokenForApp(loginUser);
        return token;
    }

    private String buildTokenByDoctor(TDoctor tDoctor) {
        if (Objects.isNull(tDoctor)) {
            return StringUtils.EMPTY;
        }
        LoginUser loginUser = new LoginUser();
        loginUser.setUserId(tDoctor.getUserId());
        SysUser user = new SysUser();
        user.setUserId(tDoctor.getUserId());
        user.setUserName(tDoctor.getPhone());
        user.setNickName(tDoctor.getName());
        user.setDelFlag(tDoctor.getDelFlag());
        user.setSex(tDoctor.getSex());
        user.setPhonenumber(tDoctor.getPhone());
        user.setDoctorId(tDoctor.getId());
        user.setHospitalId(tDoctor.getHospitalId());
        loginUser.setUser(user);
        String token = userTokenService.createTokenForApp(loginUser);
        return token;
    }
}
